"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetaTokenManager = void 0;
const AppAccessTokenHandler_1 = require("./AppAccessTokenHandler");
const ClientAccessTokenHandler_1 = require("./ClientAccessTokenHandler");
const PageAccessTokenHandler_1 = require("./PageAccessTokenHandler");
const SystemUserAccessTokenHandler_1 = require("./SystemUserAccessTokenHandler");
const UserAccessTokenHandler_1 = require("./UserAccessTokenHandler");
/**
 * Centralized manager for all Meta API token types.
 * This class provides a unified interface for handling different types of tokens
 * required to interact with various Meta APIs.
 */
class MetaTokenManager {
    /**
     * Initializes the MetaTokenManager with the necessary API client.
     * @param apiClient - The MetaApiClient instance for making API requests.
     */
    constructor(apiClient) {
        this.appTokenHandler = new AppAccessTokenHandler_1.AppAccessTokenHandler(apiClient);
        this.clientTokenHandler = new ClientAccessTokenHandler_1.ClientAccessTokenHandler(apiClient);
        this.pageTokenHandler = new PageAccessTokenHandler_1.PageAccessTokenHandler(apiClient);
        this.systemUserTokenHandler = new SystemUserAccessTokenHandler_1.SystemUserAccessTokenHandler(apiClient);
        this.userTokenHandler = new UserAccessTokenHandler_1.UserAccessTokenHandler(apiClient);
    }
    /**
     * Retrieves an app-level access token.
     * @returns A promise resolving to the app access token.
     */
    getAppToken() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.appTokenHandler.getAccessToken();
        });
    }
    /**
     * Retrieves a client-level access token.
     * @param clientToken - The client token to validate and fetch.
     * @returns A promise resolving to the validated client access token.
     */
    getClientToken(clientToken) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.clientTokenHandler.fetchToken({ clientToken });
            return this.clientTokenHandler.getAccessToken();
        });
    }
    /**
     * Retrieves a page-level access token.
     * @param userAccessToken - A user access token associated with the page.
     * @returns A promise resolving to the page access token.
     */
    getPageToken(userAccessToken) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.pageTokenHandler.fetchToken({ userAccessToken });
            return this.pageTokenHandler.getAccessToken();
        });
    }
    /**
     * Retrieves a system user access token.
     * @param systemUserId - The system user ID to fetch the token for.
     * @param appSecret - The application secret for secure token access.
     * @returns A promise resolving to the system user access token.
     */
    getSystemUserToken(systemUserId, appSecret) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.systemUserTokenHandler.fetchToken({ systemUserId, appSecret });
            return this.systemUserTokenHandler.getAccessToken();
        });
    }
    /**
     * Retrieves a user-level access token.
     * @param code - The authorization code obtained during the OAuth process.
     * @param redirectUri - The redirect URI associated with the OAuth request.
     * @returns A promise resolving to the user access token.
     */
    getUserToken(code, redirectUri) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.userTokenHandler.fetchToken({ code, redirectUri });
            return this.userTokenHandler.getAccessToken();
        });
    }
}
exports.MetaTokenManager = MetaTokenManager;
//# sourceMappingURL=MetaTokenManager.js.map