"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetaApiClient = void 0;
const MetaErrorFactory_1 = require("../errors/MetaErrorFactory");
const HelperApiClient_1 = require("../helpers/HelperApiClient");
const enum_1 = require("../enum");
/**
 * MetaApiClient class to interact with Meta APIs.
 * Handles authentication, HTTP requests, and error management.
 */
class MetaApiClient {
    /**
     * Creates an instance of MetaApiClient with an access token.
     * @param accessToken - The access token used for authenticating API requests.
     */
    constructor(config) {
        this.apiVersion = 'v12.0';
        this.apiHost = "https://graph.facebook.com";
        this.config = HelperApiClient_1.HelperApiClient.setApiClientConfig(config);
        this.accessToken = HelperApiClient_1.HelperApiClient.setAccessToken(config === null || config === void 0 ? void 0 : config.auth.accessToken);
        this.apiUrl = `${this.apiHost}/${this.apiVersion}`; // Meta Graph API base URL
    }
    /**
     * General method for making HTTP requests (GET, POST, PUT, DELETE).
     * @param method - The HTTP method (GET, POST, PUT, DELETE).
     * @param endpoint - The API endpoint to request.
     * @param params - Optional parameters to include in the request.
     * @returns A promise containing the API response data.
     */
    request(method_1, endpoint_1) {
        return __awaiter(this, arguments, void 0, function* (method, endpoint, params = {}) {
            var _a;
            let url = new URL(`${this.apiUrl}/${endpoint}`);
            params = Object.assign(Object.assign({}, params), { accessToken: this.accessToken });
            url = HelperApiClient_1.HelperApiClient.setRequestParams(url, params, this.config);
            const response = yield fetch(url.toString(), { method }); // Send the HTTP request
            // Handle HTTP errors
            if (!response.ok) {
                const errorData = yield response.json();
                MetaErrorFactory_1.MetaErrorFactory.throwRequestErrorAPI(response.status, (_a = errorData.error) === null || _a === void 0 ? void 0 : _a.message);
            }
            const data = yield response.json(); // Parse the response JSON
            return { data }; // Return the data from the API
        });
    }
    /**
     * Makes a GET request to the specified endpoint.
     * @param endpoint - The API endpoint to request.
     * @param params - Optional parameters to include in the request.
     * @returns A promise containing the API response data.
     */
    get(endpoint, params = {}) {
        return this.request(enum_1.HttpMethods.GET, endpoint, params);
    }
    /**
     * Makes a POST request to the specified endpoint.
     * @param endpoint - The API endpoint to request.
     * @param params - Optional parameters to include in the request.
     * @returns A promise containing the API response data.
     */
    post(endpoint, params = {}) {
        return this.request(enum_1.HttpMethods.POST, endpoint, params);
    }
    /**
     * Makes a PUT request to the specified endpoint.
     * @param endpoint - The API endpoint to request.
     * @param params - Optional parameters to include in the request.
     * @returns A promise containing the API response data.
     */
    put(endpoint, params = {}) {
        return this.request(enum_1.HttpMethods.PUT, endpoint, params);
    }
    /**
     * Makes a DELETE request to the specified endpoint.
     * @param endpoint - The API endpoint to request.
     * @param params - Optional parameters to include in the request.
     * @returns A promise containing the API response data.
     */
    delete(endpoint, params = {}) {
        return this.request(enum_1.HttpMethods.DELETE, endpoint, params);
    }
    /**
     * Get configuration options
     */
    get configOptions() {
        return this.config;
    }
    /**
     * Refreshes the access token.
     * @returns A promise containing the new access token.
     * @throws Throws an error if token refresh fails.
     */
    refreshToken() {
        return __awaiter(this, void 0, void 0, function* () {
            // Example URL for token refresh
            const url = `${this.apiUrl}/oauth/access_token?grant_type=refresh_token&refresh_token=${this.accessToken}`;
            const response = yield fetch(url); // Send the refresh token request
            const data = yield response.json(); // Parse the response JSON
            if (!response.ok || !data.access_token) {
                throw new Error('Failed to refresh access token'); // Error if no new token is returned
            }
            this.accessToken = data.access_token; // Update the access token
            return this.accessToken; // Return the new token
        });
    }
}
exports.MetaApiClient = MetaApiClient;
//# sourceMappingURL=MetaApiClient.js.map